@@ -2,7 +2,7 @@
  *
  * MIT License
  *
- * Copyright (c) 2023 Advanced Micro Devices, Inc.
+ * Copyright (c) 2025 Advanced Micro Devices, Inc.
  *
  * Permission is hereby granted, free of charge, to any person obtaining a copy
  * of this software and associated documentation files (the "Software"), to deal
@@ -94,11 +94,11 @@
 // by default DL kernels are turned OFF
 //
 #ifndef CK_ENABLE_DL_KERNELS
-/* #undef CK_ENABLE_DL_KERNELS */
+#define CK_ENABLE_DL_KERNELS ON
 #endif
 
 #ifndef CK_ENABLE_DPP_KERNELS
-/* #undef CK_ENABLE_DPP_KERNELS */
+#define CK_ENABLE_DPP_KERNELS ON
 #endif
 
 //
@@ -115,6 +115,10 @@
 #define CK_USE_WMMA ON
 #endif
 
+#ifndef CK_USE_WMMA_FP8
+#define CK_USE_WMMA_FP8 ON
+#endif
+
 #ifndef CK_USE_GFX94
 #define CK_USE_GFX94 ON
 #endif
